# rmg/fb apr2011: added a function for shutter control via champ server

import socket
import numarray as na
import struct
import common

class IRCamCommands:
    def __init__(self):
        """ constructor, setting initial variables """
        self.rows =16
        self.cols = 16
        self.rowOffset = 7
        self.colOffset = 20
        self.nreads =10 
        self.numberOfCoadds = 1
        self.framesPerReset = 21
        self.expTime = 0
        self.numberOfFrames = 42
        self.byteSwapping  =False
        self.doReset = True
        self.doDeinterlace = False
        self.doImageSave = False
        self.daqRunning = False
        self.read = False
        self.voffset = 4.6 
        self.vreset = 0.5
        self.vd = 5.0
        self.ictl = 3.3
        self.continuousReadout = True
	self.savenum = 0 
	self.savesPerFile = 100 
	self.numBkgnd = 100 
	self.numberCoherent = 1

        self.imageDirectory = '/data/'
        self.imageFilename = 'champ.fit'
#        self.opdmaps = 'jdm_opds.txt'
        self.useBackground = False
        self.numberOfBackgroundFrames = 1000

    def initSocket(self):
        self.sock = None
        for res in socket.getaddrinfo(common.host, common.port, socket.AF_UNSPEC, socket.SOCK_STREAM):
            af, socktype, proto, canonname, sa = res
            try:
                self.sock = socket.socket(af, socktype, proto)
            except socket.error, msg:
                self.sock = None
                continue
            try:
                self.sock.connect(sa)
                self.fd = self.sock.fileno()
            except socket.error, msg:
                self.sock.close()
                self.sock = None
                continue
            break
        if self.sock is None:
            print 'could not open socket'

    ## rmg/fb april 2011 - tells champ server to open only one shutter    
    def openShutter(self, number):
        self.initSocket()
        self.sock.settimeout(10)
        self.sock.send('UX')
        print "sent UX..."
        #ack= self.sock.recv(10)
        #print 'ack = ', ack
        #if ack != 'OK':
        #    print 'Failed to receive OK from server'
        self.sock.send(struct.pack('L',number))
        print "sent beam number...", number
        #ack= self.sock.recv(10)
        #print 'ack = ', ack
        #if ack != 'OK':
        #    print 'Failed to receive OK from server'
        # do we need to close the socket??    

            
    def startExposure(self):
	print "Sent Structure, now starting Exposure"
        self.initSocket()
        self.sock.settimeout(10)
        self.sock.send('SX')
        print "sent SX.."

        print "sent data"
        ack= self.sock.recv(10)
        print 'ack = ', ack
        if ack != 'OK':
            print 'Failed to receive OK from server'

    def stopExposure(self):
        self.initSocket()
        self.sock.settimeout(5)
        self.sock.send('XX')
        print "sent data"
        ack= self.sock.recv(2)
        print 'ack = ', ack
        if ack != 'OK':
            print 'Failed to receive OK from server'

    def getBackgroundFrame(self):
        self.initSocket()
        self.sock.settimeout(5)
        self.sock.send('TB')
        print "sent data"
        ack= self.sock.recv(2)
        print 'ack = ', ack
        if ack != 'OK':
            print 'Failed to receive OK from server'

    def resetBackgroundFrame(self):
        self.initSocket()
        self.sock.settimeout(5)
        self.sock.send('RB')
        print "sent data"
        ack= self.sock.recv(2)
        print 'ack = ', ack
        if ack != 'OK':
            print 'Failed to receive OK from server'

    def useBackgroundFrame(self):
        self.initSocket()
        self.sock.settimeout(5)
        self.sock.send('UB')
        print "sent data"
        ack= self.sock.recv(2)
        print 'ack = ', ack
        if ack != 'OK':
            print 'Failed to receive OK from server'



    def resetCont(self):
        self.initSocket()
        self.sock.send('IO')
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def setBias(self):
        self.initSocket()
        self.sock.send('SB')
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def setCont(self):
        self.initSocket()
        self.sock.send('SC')
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def updateDaqStructure(self):
        self.initSocket()
        c_struct = struct.pack(
            '8HL6B4f30s20sB4H',
            self.rows,
            self.cols,
            self.rowOffset,
            self.colOffset,
            self.nreads,
            self.numberOfCoadds,
            self.framesPerReset,
            self.expTime,
            self.numberOfFrames,
            self.byteSwapping,
            self.doReset,
            self.doDeinterlace,
            self.doImageSave,
            self.daqRunning,
            self.read,
            self.voffset,
            self.vreset,
            self.vd,
            self.ictl,
            self.imageDirectory,
            self.imageFilename,
            self.continuousReadout,
            self.savenum,
            self.savesPerFile,
            self.numBkgnd,
            self.numberCoherent
            )
        
        strLen = len(c_struct)
        print strLen
        self.sock.send('CN')
        print "sent CN"
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(c_struct)
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)
        
    def updateDoImageSave(self):
        self.initSocket()
        c_struct = struct.pack(
            '8HL6B4f30s20sB4H',
            self.rows,
            self.cols,
            self.rowOffset,
            self.colOffset,
            self.nreads,
            self.numberOfCoadds,
            self.framesPerReset,
            self.expTime,
            self.numberOfFrames,
            self.byteSwapping,
            self.doReset,
            self.doDeinterlace,
            self.doImageSave,
            self.daqRunning,
            self.read,
            self.voffset,
            self.vreset,
            self.vd,
            self.ictl,
            self.imageDirectory,
            self.imageFilename,
            self.continuousReadout,
            self.savenum,
            self.savesPerFile,
            self.numBkgnd,
	    self.numberCoherent
            )
        
        strLen = len(c_struct)
        print strLen
        self.sock.send('SD')
        print "sent SD"
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(c_struct)
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)
        

        
    def sendStruct(self):
        self.initSocket()
        c_struct = struct.pack(
            '8HL6B4f30s20sB4H',
            self.rows,
            self.cols,
            self.rowOffset,
            self.colOffset,
            self.nreads,
            self.numberOfCoadds,
            self.framesPerReset,
            self.expTime,
            self.numberOfFrames,
            self.byteSwapping,
            self.doReset,
            self.doDeinterlace,
            self.doImageSave,
            self.daqRunning,
            self.read,
            self.voffset,
            self.vreset,
            self.vd,
            self.ictl,
            self.imageDirectory,
            self.imageFilename,
            self.continuousReadout,
	    self.savenum,
	    self.savesPerFile,
	    self.numBkgnd,
            self.numberCoherent
            )
        
        strLen = len(c_struct)
        print strLen
        self.sock.send('AQ')
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(c_struct)
        data = self.sock.recv(2)
        self.sock.close()
        print '(sendStruct)Received', repr(data)

    def reOPLE(self):
        self.initSocket()
        self.sock.send('RO')
        self.sock.send(struct.pack('L',1))
        self.sock.send(struct.pack('H',0))
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def killServer(self):
        self.initSocket()
        self.sock.send('EN')
        self.sock.send(struct.pack('L',1))
        self.sock.send(struct.pack('H',0))
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

        

    def sendToOple(self):
        self.initSocket()
        self.sock.settimeout(5)
        self.sock.send('OP')
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def getDelayLinesOffsets(self):
        self.initSocket()
        self.sock.settimeout(5)
        strLen = 48
        self.sock.send('OF')
        self.sock.send(struct.pack('L',strLen))
        data = self.sock.recv(strLen)
        self.sock.close()
        common.delayLinesOffsets = na.fromstring(data, na.Float64, 6)

    def getFringeInfo(self):
        self.initSocket()
        self.sock.settimeout(5)
        strLen = 48
        self.sock.send('FI')
        self.sock.send(struct.pack('L',strLen))
        data = self.sock.recv(strLen)
        self.sock.close()
        common.opds = na.fromstring(data, na.Float32, (2, 6))[0]
        common.weights = na.fromstring(data, na.Float32, (2, 6))[1]

    def sendGdtInfo(self):
        self.initSocket()
        self.sock.settimeout(5)
##         print "common.gdtInfo = ", common.gdtInfo
        gdtStruct = struct.pack(
            '7I',common.gdtInfo[0]
            ,common.gdtInfo[1]
            ,common.gdtInfo[2]
            ,common.gdtInfo[3]
            ,common.gdtInfo[4]
            ,common.gdtInfo[5]
            ,common.gdtInfo[6])
        strLen = len(gdtStruct)
##         print "strLen = ", strLen
        self.sock.send('GI')
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(gdtStruct)
        data = self.sock.recv(2)
        self.sock.close()
##         print 'Received', repr(data)

    def sendUpdatedOffsets(self):
        self.initSocket()
        self.sock.settimeout(5)
##         print "common.updatedOffsets = ", common.updatedOffsets
        opdStruct = struct.pack(
            '6d',common.updatedOffsets[0]
            ,common.updatedOffsets[1]
            ,common.updatedOffsets[2]
            ,common.updatedOffsets[3]
            ,common.updatedOffsets[4]
            ,common.updatedOffsets[5]
            )
        strLen = len(opdStruct)
##         print "strLen = ", strLen
        self.sock.send('OP')
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(opdStruct)
        data = self.sock.recv(2)
        self.sock.close()
##         print 'Received', repr(data)
